package dev.shadowsoffire.apotheosis.spawn.spawner;

import net.minecraft.world.level.block.Blocks;
import net.minecraft.util.RandomSource;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.Item;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.ChatFormatting;
import java.util.function.Consumer;
import dev.shadowsoffire.apotheosis.spawn.modifiers.SpawnerStats;
import java.util.Objects;
import net.minecraft.client.gui.screens.Screen;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.network.chat.Component;
import net.minecraft.server.level.ServerPlayer;
import dev.shadowsoffire.apotheosis.advancements.AdvancementTriggers;
import dev.shadowsoffire.apotheosis.spawn.modifiers.SpawnerModifier;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.level.storage.loot.parameters.LootContextParams;
import java.util.List;
import net.minecraft.world.level.storage.loot.LootParams;
import net.minecraft.stats.Stats;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.item.enchantment.Enchantments;
import dev.shadowsoffire.apotheosis.spawn.SpawnerModule;
import javax.annotation.Nullable;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.nbt.Tag;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.player.Player;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.level.block.SoundType;
import net.minecraft.world.level.material.MapColor;
import net.minecraft.world.level.block.state.BlockBehaviour;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.StateDefinition;
import dev.shadowsoffire.placebo.util.IReplacementBlock;
import net.minecraft.world.level.block.SpawnerBlock;

public class ApothSpawnerBlock extends SpawnerBlock implements IReplacementBlock
{
    private static final ApothSpawnerTile tooltipTile;
    protected StateDefinition<Block, BlockState> container;
    
    public ApothSpawnerBlock() {
        super(BlockBehaviour.Properties.m_284310_().m_284180_(MapColor.f_283947_).m_60999_().m_60978_(5.0f).m_60918_(SoundType.f_56743_).m_60955_());
    }
    
    public ItemStack getCloneItemStack(final BlockState state, final HitResult target, final BlockGetter world, final BlockPos pos, final Player player) {
        final ItemStack s = new ItemStack((ItemLike)this);
        final BlockEntity te = world.m_7702_(pos);
        if (te != null) {
            s.m_41784_().m_128365_("BlockEntityTag", (Tag)te.m_187482_());
        }
        return s;
    }
    
    public void m_6402_(final Level world, final BlockPos pos, final BlockState state, final LivingEntity placer, final ItemStack stack) {
        final BlockEntity te = world.m_7702_(pos);
        if (te != null && stack.m_41782_()) {
            te.m_142466_(stack.m_41698_("BlockEntityTag"));
        }
    }
    
    public BlockEntity m_142194_(final BlockPos pPos, final BlockState pState) {
        return (BlockEntity)new ApothSpawnerTile(pPos, pState);
    }
    
    public void m_6240_(final Level world, final Player player, final BlockPos pos, final BlockState state, @Nullable final BlockEntity te, final ItemStack stack) {
        super.m_6240_(world, player, pos, state, te, stack);
        if (SpawnerModule.spawnerSilkLevel != -1 && stack.getEnchantmentLevel(Enchantments.f_44985_) >= SpawnerModule.spawnerSilkLevel) {
            if (SpawnerModule.spawnerSilkDamage > 1) {
                player.m_21205_().m_41622_(SpawnerModule.spawnerSilkDamage - 1, (LivingEntity)player, pl -> pl.m_21166_(EquipmentSlot.MAINHAND));
            }
            player.m_36246_(Stats.f_12949_.m_12902_((Object)this));
            player.m_36399_(0.035f);
        }
    }
    
    @Deprecated
    public List<ItemStack> m_49635_(final BlockState state, final LootParams.Builder params) {
        final ItemStack tool = (ItemStack)params.m_287261_(LootContextParams.f_81463_);
        if (SpawnerModule.spawnerSilkLevel != -1 && tool.getEnchantmentLevel(Enchantments.f_44985_) >= SpawnerModule.spawnerSilkLevel) {
            final ItemStack s = new ItemStack((ItemLike)this);
            final BlockEntity te = (BlockEntity)params.m_287159_(LootContextParams.f_81462_);
            if (te != null) {
                s.m_41784_().m_128365_("BlockEntityTag", (Tag)te.m_187482_());
            }
            return List.of(s);
        }
        return super.m_49635_(state, params);
    }
    
    public InteractionResult m_6227_(final BlockState state, final Level world, final BlockPos pos, final Player player, final InteractionHand hand, final BlockHitResult hit) {
        final BlockEntity te = world.m_7702_(pos);
        final ItemStack stack = player.m_21120_(hand);
        final ItemStack otherStack = player.m_21120_((hand == InteractionHand.MAIN_HAND) ? InteractionHand.OFF_HAND : InteractionHand.MAIN_HAND);
        if (te instanceof final ApothSpawnerTile tile) {
            final SpawnerModifier match = SpawnerModifier.findMatch(tile, stack, otherStack);
            if (match != null && match.apply(tile)) {
                if (world.f_46443_) {
                    return InteractionResult.SUCCESS;
                }
                if (!player.m_7500_()) {
                    stack.m_41774_(1);
                    if (match.consumesOffhand()) {
                        otherStack.m_41774_(1);
                    }
                }
                AdvancementTriggers.SPAWNER_MODIFIER.trigger((ServerPlayer)player, tile, match);
                world.m_7260_(pos, state, state, 3);
                return InteractionResult.SUCCESS;
            }
        }
        return InteractionResult.PASS;
    }
    
    public void m_5871_(final ItemStack stack, final BlockGetter worldIn, final List<Component> tooltip, final TooltipFlag flagIn) {
        if (stack.m_41782_() && stack.m_41783_().m_128425_("BlockEntityTag", 10)) {
            if (Screen.m_96638_()) {
                final CompoundTag tag = stack.m_41783_().m_128469_("BlockEntityTag");
                ApothSpawnerBlock.tooltipTile.m_142466_(tag);
                final ApothSpawnerTile tooltipTile = ApothSpawnerBlock.tooltipTile;
                Objects.requireNonNull(tooltip);
                SpawnerStats.generateTooltip(tooltipTile, (Consumer)tooltip::add);
            }
            else {
                tooltip.add((Component)Component.m_237115_("misc.apotheosis.shift_stats").m_130940_(ChatFormatting.GRAY));
            }
        }
    }
    
    public static Component concat(final Object... args) {
        return (Component)Component.m_237110_("misc.apotheosis.value_concat", new Object[] { args[0], Component.m_237113_(args[1].toString()).m_130940_(ChatFormatting.GRAY) }).m_130940_(ChatFormatting.GREEN);
    }
    
    public Item m_5456_() {
        return Items.f_42007_;
    }
    
    public int getExpDrop(final BlockState state, final LevelReader world, final RandomSource randomSource, final BlockPos pos, final int fortune, final int silktouch) {
        return (silktouch == 0) ? super.getExpDrop(state, world, randomSource, pos, fortune, silktouch) : 0;
    }
    
    public void _setDefaultState(final BlockState state) {
        this.m_49959_(state);
    }
    
    public void setStateContainer(final StateDefinition<Block, BlockState> container) {
        this.container = container;
    }
    
    public StateDefinition<Block, BlockState> m_49965_() {
        return (StateDefinition<Block, BlockState>)((this.container == null) ? super.m_49965_() : this.container);
    }
    
    static {
        tooltipTile = new ApothSpawnerTile(BlockPos.f_121853_, Blocks.f_50016_.m_49966_());
    }
}
